from abc import ABC, abstractmethod


class Person:
    def __init__(self, name):
        self.name = name
        self.db = PersonDB()      # 기본 db 설정
        

    def __repr__(self):
        return f'Person(name={self.name})'
    
    def save(self):
        self.db.save(self)


class PersonStorage(ABC):
    @abstractmethod
    def save(self, person):
        pass


class PersonDB(PersonStorage):
    def save(self, person):
        print(f'Save the {person} to database')


class PersonJSON(PersonStorage):
    def save(self, person):
        print(f'Save the {person} to a JSON file')


class PersonXML(PersonStorage):
    def save(self, person):
        print(f'Save the {person} to a XML file')


if __name__ == '__main__':
    person = Person('John Doe')
    storage = PersonXML()
    storage.save(person)
    
    p = Person('blackPink')
    p.save()
    